@@ -44,13 +44,15 @@ const gchar *kbType_ids[]={
 };
 
 
-#define SELKEYS_ARRAY_SIZE 7
+#define SELKEYS_ARRAY_SIZE 8
 const gchar *selKeys_array[SELKEYS_ARRAY_SIZE+1]={
     "1234567890",
     "asdfghjkl;",
     "asdfzxcv89",
     "asdfjkl789",
-    "aoeuhtn789",
+    "aoeu;qjkix", /* Dvorak */
+    "aoeuhtnsid", /* Dvorak */
+    "aoeuidhtns", /* Dvorak */
     "1234qweras",
     NULL
 };
@@ -58,7 +60,7 @@ const gchar *selKeys_array[SELKEYS_ARRAY_SIZE+1]={
 const gchar *syncCapsLockLocal_strs[]={
     NC_("Sync","disable"),
     NC_("Sync","keyboard"),
-    NC_("Sync","im"),
+    NC_("Sync","input method"),
     NULL
 };
 
@@ -69,6 +71,13 @@ const gchar *outputCharsets[]={
     NULL
 };
 
+const gchar *inputStyles[]={
+    N_("in application window"),
+    N_("in candidate window"),
+    NULL
+};
+
+
 #ifdef IBUS_CHEWING_MAIN
 static ChewingKbType kbType_id_get_index(const gchar *kbType_id){
     ChewingKbType i=0;
@@ -96,7 +105,7 @@ static void selKeys_set_callback(PropertyContext *ctx, GValue *value){
     IBusChewingEngine *engine=(IBusChewingEngine *) ctx->userData;
     ibus_chewing_engine_set_selKeys_string(engine,g_value_get_string(value));
     if (!engine->table){
-	engine->table=ibus_lookup_table_new(strlen(g_value_get_string(value)),0,FALSE,TRUE);
+	engine->table=g_object_ref_sink(ibus_lookup_table_new(strlen(g_value_get_string(value)),0,FALSE,TRUE));
     }
     ibus_chewing_engine_set_lookup_table_label(engine,g_value_get_string(value));
 #endif
@@ -127,7 +136,11 @@ static void easySymbolInput_set_callback(PropertyContext *ctx, GValue *value){
 #ifdef IBUS_CHEWING_MAIN
     IBusChewingEngine *engine=(IBusChewingEngine *) ctx->userData;
     chewing_set_easySymbolInput(engine->context,(g_value_get_boolean(value)) ? 1: 0);
-    engine->_priv->easySymbolInput=g_value_get_boolean(value);
+    if (g_value_get_boolean(value)){
+	engine->flags |= CHEWING_FLAG_EASY_SYMBOL_INPUT;
+    }else{
+	engine->flags &= ~CHEWING_FLAG_EASY_SYMBOL_INPUT;
+    }
 #endif
 }
 
@@ -149,7 +162,11 @@ static void maxChiSymbolLen_set_callback(PropertyContext *ctx, GValue *value){
 static void forceLowercaseEnglish_set_callback(PropertyContext *ctx, GValue *value){
 #ifdef IBUS_CHEWING_MAIN
     IBusChewingEngine *engine=(IBusChewingEngine *) ctx->userData;
-    engine->forceLowercaseEnglish=g_value_get_boolean(value);
+    if (g_value_get_boolean(value)){
+	engine->flags |= CHEWING_FLAG_FORCE_LOWERCASE_ENGLISH;
+    }else{
+	engine->flags &= ~CHEWING_FLAG_FORCE_LOWERCASE_ENGLISH;
+    }
 #endif
 }
 
@@ -159,7 +176,7 @@ static void syncCapsLockLocal_set_callback(PropertyContext *ctx, GValue *value){
     const gchar *str=g_value_get_string(value);
     if (strcmp(str,"keyboard")==0){
 	engine->syncCapsLock_local=CHEWING_MODIFIER_SYNC_FROM_KEYBOARD;
-    }else if (strcmp(str,"im")==0){
+    }else if (strcmp(str,"input method")==0){
 	engine->syncCapsLock_local=CHEWING_MODIFIER_SYNC_FROM_IM;
     }else{
 	engine->syncCapsLock_local=CHEWING_MODIFIER_SYNC_DISABLE;
@@ -170,7 +187,26 @@ static void syncCapsLockLocal_set_callback(PropertyContext *ctx, GValue *value){
 static void numpadAlwaysNumber_set_callback(PropertyContext *ctx, GValue *value){
 #ifdef IBUS_CHEWING_MAIN
     IBusChewingEngine *engine=(IBusChewingEngine *) ctx->userData;
-    engine->numpadAlwaysNumber=g_value_get_boolean(value);
+    if (g_value_get_boolean(value)){
+	engine->flags |= CHEWING_FLAG_NUMPAD_ALWAYS_NUMBER;
+    }else{
+	engine->flags &= ~CHEWING_FLAG_NUMPAD_ALWAYS_NUMBER;
+    }
+#endif
+}
+
+static void inputStyle_set_callback(PropertyContext *ctx, GValue *value){
+#ifdef IBUS_CHEWING_MAIN
+    IBusChewingEngine *engine=(IBusChewingEngine *) ctx->userData;
+    const gchar *str=g_value_get_string(value);
+    if (strcmp(str,"in application window")==0){
+	engine->_priv->inputStyle=CHEWING_INPUT_STYLE_IN_APPLICATION;
+    }else  if (strcmp(str,"in candidate window")==0){
+	engine->_priv->inputStyle=CHEWING_INPUT_STYLE_IN_CANDIDATE;
+    }else{
+	engine->_priv->inputStyle=CHEWING_INPUT_STYLE_IN_APPLICATION;
+    }
+    ibus_chewing_engine_force_commit(engine);
 #endif
 }
 
@@ -182,7 +218,7 @@ static void candPerPage_set_callback(PropertyContext *ctx, GValue *value){
 	ibus_lookup_table_clear(engine->table);
 	engine->table->page_size=g_value_get_int(value);
     }else{
-	engine->table=ibus_lookup_table_new(g_value_get_int(value),0,FALSE,TRUE);
+	engine->table=g_object_ref_sink(ibus_lookup_table_new(g_value_get_int(value),0,FALSE,TRUE));
     }
 #endif
 }
@@ -204,7 +240,11 @@ static void spaceAsSelection_set_callback(PropertyContext *ctx, GValue *value){
 static void plainZhuyin_set_callback(PropertyContext *ctx, GValue *value){
 #ifdef IBUS_CHEWING_MAIN
     IBusChewingEngine *engine=(IBusChewingEngine *) ctx->userData;
-    engine->plainZhuyin=g_value_get_boolean(value);
+    if (g_value_get_boolean(value)){
+	engine->flags |= CHEWING_FLAG_PLAIN_ZHUYIN;
+    }else{
+	engine->flags &= ~CHEWING_FLAG_PLAIN_ZHUYIN;
+    }
 #endif
 }
 
@@ -215,7 +255,7 @@ PropertySpec propSpecs[]={
 	"default", kbType_ids, NULL, 0, 0,
 	NULL, KBType_set_callback,
 	MAKER_DIALOG_PROPERTY_FLAG_INEDITABLE | MAKER_DIALOG_PROPERTY_FLAG_HAS_TRANSLATION, 0, 0,
-	N_("Select keyboard layout of Zhuyin symbols."),
+	N_("Select Zhuyin keyboard layout."),
     },
     {G_TYPE_STRING, "selKeys", "Keyboard",  N_("Selection keys"),
 	"1234567890", selKeys_array, NULL, 0, 0,
@@ -279,7 +319,7 @@ It is handy if you wish to enter lowercase by default. Uppercase can still be in
 this option determines how these status be synchronized. Valid values:\n\
 \"disable\": Do nothing.\n\
 \"keyboard\": IM status follows keyboard status.\n\
-\"im\": Keyboard status follows IM status."),
+\"IM\": Keyboard status follows IM status."),
     },
 
     {G_TYPE_BOOLEAN, "numpadAlwaysNumber", "Editing", N_("Number pad always input number"),
@@ -289,6 +329,16 @@ this option determines how these status be synchronized. Valid values:\n\
 	N_("Always input numbers when number keys from key pad is inputted."),
     },
 
+    {G_TYPE_STRING, "inputStyle", "Editing", N_("Input Style"),
+	"in candidate window", inputStyles, NULL,  0, 1,
+	NULL, inputStyle_set_callback,
+	MAKER_DIALOG_PROPERTY_FLAG_INEDITABLE | MAKER_DIALOG_PROPERTY_FLAG_HAS_TRANSLATION,
+       	0, 0,
+	N_("Input style determines where the preedit strings be shown and edited.\n"
+		"\"in application window\": Preedit strings are edited on the target application window.\n"
+		"\"in candidate window\": Preedit strings are edited on the candidate selection window."),
+    },
+
     {G_TYPE_BOOLEAN, "plainZhuyin", "Selecting", N_("Plain Zhuyin mode"),
 	"0", NULL, NULL, 0, 1,
 	NULL, plainZhuyin_set_callback,
